package com.xyzc.start.service.impl;

import com.xyzc.common.core.exception.ErrorCode;
import com.xyzc.common.core.exception.ServerException;
import com.xyzc.common.core.utils.AssertUtils;
import com.xyzc.common.web.security.user.SecurityUser;
import com.xyzc.start.domain.BaseUser;
import com.xyzc.start.domain.bo.AgentCardSecretBo;
import com.xyzc.start.domain.bo.HobbyBo;
import com.xyzc.start.domain.vo.*;
import com.xyzc.start.mapper.*;
import com.xyzc.start.service.IUserInfoService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;


@Service
@RequiredArgsConstructor
public class UserInfoServiceImpl implements IUserInfoService {

    private final BaseUserMapper baseUserMapper;
    private final IndustryMapper industryMapper;
    private final BaseUserPhotoFolderMapper photoFolderMapper;

    private final FuelFlowMapper fuelFlowMapper;

    private final AppMapper appMapper;

    /**
     * 获取当前用户信息
     *
     * @return
     */
    @Override
    public BaseUserVo queryCurrent() {
        BaseUserVo baseUser = baseUserMapper.queryCurrentInfo(SecurityUser.getUserId());
        // todo 后期加入次数
        return baseUser;
    }

    /**
     * 保存用户爱好
     *
     * @param hobbyBo HobbyBo
     */
    @Override
    public void saveHobby(HobbyBo hobbyBo) {

        baseUserMapper.saveHobbys(SecurityUser.getUserId(), hobbyBo);
    }


    /**
     * 获取用户信息
     *
     * @return
     */
    @Override
    public BaseUserVo queryInfo() {
        BaseUserVo baseUserVo = baseUserMapper.queryBaseUserVoById(SecurityUser.getUserId());

        // TODO： 后续计算
        baseUserVo.setUseCount(108);

        // 添加用户拥有的会员卡
        List<BaseUserMemberCardVo> memberCardVos = baseUserMapper.queryMyMemberCardByUserId(baseUserVo.getId());
        baseUserVo.setMemberCardVoList(memberCardVos);

        // 添加用户会员卡数量
        baseUserVo.setMemberCardCount(memberCardVos.size());


        return baseUserVo;
    }

    /**
     * 获取所有的行业
     *
     * @return
     */
    @Override
    public List<IndustryVo> getIndustryAll() {

        return industryMapper.queryList(null);
    }

    /**
     * 根据 id获取用户图画
     *
     * @param id Long
     * @return BaseUserPhotoFolderVo
     */
    @Override
    public BaseUserPhotoFolderVo queryPhotoById(Long id) {
        return photoFolderMapper.queryPhotoFolderById(SecurityUser.getUserId(), id);
    }

    /**
     * 获取用户画夹
     *
     * @param id
     * @return
     */
    @Override
    public List<BaseUserPhotoFolderVo> photoFolder(Long id) {
        return photoFolderMapper.queryPhotoFolder(SecurityUser.getUserId(), id);
    }

    /**
     * @param id
     */
    @Override
    public void deletePhoto(Long id) {
        //TODO 记得后期删除图片
        Boolean flag = photoFolderMapper.deletePhotoById(SecurityUser.getUserId(), id);
        if (!flag) {
            throw new ServerException(ErrorCode.USER_DELETE_IMAGE_ERROR);
        }
    }

    /**
     * @param ids
     */
    @Override
    public void deletePhotos(List<Long> ids) {
//        photoFolderMapper.deletePhotosByIds(SecurityUser.getUserId(), ids);
    }

    /**
     * 获取用户燃料流水
     */
    @Override
    public List<BaseUserFuelFlowVo> queryFuelFlow(Long flowId) {

        return fuelFlowMapper.queryFuelFlowList(SecurityUser.getUserId(), flowId);
    }

    /**
     * 销卡
     *
     * @param exchangeBo
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void exchange(AgentCardSecretBo exchangeBo) {
        // TODO： 注意，这里要限制用户每天最多兑换10次，输入10次锁住不让再兑换，并且记录重要事件日志

        //销卡的步骤
        // 1. 查询该卡密是否有效，如果无效，则抛出异常


        // 2. 查询该卡密是否使用过，如果使用过，则抛出异常

        // 3. 查询该卡密是否属于该用户所属的代理商，如果不属于，则抛出异常

        // 4. 开始核销卡密，先将其状态修改为 正在核销

    }

    /**
     * 根据应用id扣除对应的燃料
     *
     * @param appId
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deduct(Long appId) {
        // 首先获取个人燃料剩余量
        BaseUser baseUser = baseUserMapper.selectVoByIdForUpdate(SecurityUser.getUserId());

        // 然后获取应用对应的ai模型燃料扣除量
        AiModelVo aiModelVo = appMapper.queryAiModelFuel(appId);
        baseUser.setSurplusFuel(baseUser.getSurplusFuel()-aiModelVo.getConsumeFuelNumber());

        // 判断扣除后的用户剩余燃料是否小于0，如果小于0，则抛出异常
        AssertUtils.surplusFuel(baseUser.getSurplusFuel());

        // 然后扣除燃料返回成功
        baseUserMapper.updateById(baseUser);
    }
}

